import os
import subprocess
from typing import Any

from hatchling.builders.hooks.plugin.interface import BuildHookInterface  # type: ignore


class BuildFrontend(BuildHookInterface):
    def initialize(self, version: str, build_data: dict[str, Any]) -> None:
        # Only build the front end once, in the source dist, the wheel build just copies it from there
        if self.target_name == "sdist":
            if not os.environ.get("SKIP_PRE_BUILD"):
                print("Building front end...")
                try:
                    subprocess.check_output("yarn install", cwd="locust/webui", shell=True, stderr=subprocess.STDOUT)
                    subprocess.check_output("yarn build", cwd="locust/webui", shell=True, stderr=subprocess.STDOUT)
                except subprocess.CalledProcessError as e:
                    raise RuntimeError(f"'{e.cmd}' got exit code {e.returncode}: {e.output}")

                return super().initialize(version, build_data)



import os


def get_abspaths_in(path, extension=None):
    return [
        os.path.abspath(os.path.join(root, f))
        for root, _dirs, fs in os.walk(path)
        for f in fs
        if os.path.isfile(os.path.join(root, f))
        and (f.endswith(extension) or extension is None)
        and not f.startswith("_")
    ]



from datetime import datetime, timezone


def format_utc_timestamp(unix_timestamp):
    return datetime.fromtimestamp(int(unix_timestamp), timezone.utc).strftime("%Y-%m-%dT%H:%M:%SZ")


def format_safe_timestamp(unix_timestamp):
    return datetime.fromtimestamp(int(unix_timestamp)).strftime("%Y-%m-%d-%Hh%M")


def format_duration(start_time, end_time):
    seconds = int(end_time) - int(start_time)
    days = seconds // 86400
    hours = (seconds % 86400) // 3600
    minutes = (seconds % 3600) // 60
    seconds = seconds % 60

    time_parts = [(days, "day"), (hours, "hour"), (minutes, "minute"), (seconds, "second")]

    parts = [f"{value} {label}{'s' if value != 1 else ''}" for value, label in time_parts if value > 0]

    return " and ".join(filter(None, [", ".join(parts[:-1])] + parts[-1:])) if parts else "0 seconds"



from locust import User, events

import time

import psycopg


class PostgresClient:
    def __init__(self, conn_string):
        self.connection = psycopg.connect(conn_string)

    def execute_query(self, query):
        start_time = time.time()
        try:
            self.connection.execute(query)
            response_time = int((time.time() - start_time) * 1000)
            events.request.fire(
                request_type="postgres_success",
                name="execute_query",
                response_time=response_time,
                response_length=0,
            )
        except Exception as e:
            response_time = int((time.time() - start_time) * 1000)
            events.request.fire(
                request_type="postgres_failure",
                name="execute_query",
                response_time=response_time,
                response_length=0,
                exception=e,
            )

    def close(self):
        self.connection.close()


class PostgresUser(User):
    abstract = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.client = PostgresClient(conn_string=self.conn_string)

    def on_stop(self):
        self.client.close()
